pro get_ellipse_roi, struct_in, data_in, _extra=e, lat0=lat0, lon0=lon0, $
  radang=radang, oplot=oplot, display=display, noerase=noerase, $
  ss_roi=ss_roi, occult=occult, qstop=qstop, data_roi_only=data_roi_only

;+
;   Name: get_ellipse_roi
;
;   Purpose: Determine device coordinates of points within a circle, seen
;            in projection, of specified angular radius located at any
;            specified position (lon,lat) on the surface of a sphere.
;
;   Input Parameters:
;	struct_in - IDL structure containing image parameters
;		    (image time, image size, pixel size, pointing)
;       lat0      - Latitude (deg) of center of the circle
;       lon0      - Longitude (deg) of the center of the circle
;       radang    - Radius of the circle in degrees of arc measured from the
;                   center of the sphere
;   Optional Input Parameters:
;	data_in   - Image array (for display purposes only)
;       oplot     - If set, then do not re-draw the sphere but simply overplot
;                   the circle
;       noerase   - If set, do not erase viewing window prior to drawing sphere
;
;   Output Parameters:
;       ss_roi    - Device coordinates of image points within boundary of projected
;		    circle
;       occult    - Set if any part of the the projected circle goes over the
;                   limb
;
;   Calling Sequence:
;	get_ellipse_roi, struct_in, ss_roi=ss_roi [, data_in, _extra=e, lat0=lat0, $
;	  lon0=lon0, radang=radang, oplot=oplot, display=display, noerase=noerase, $
;	  occult=occult]
;
;   History:
;       25-Apr-2001 - Written by Greg Slater
;-

if (keyword_set(latdel) and keyword_set(lats)) eq 0 then latdel = 15
if (keyword_set(londel) and keyword_set(lons)) eq 0 then londel = 15
if keyword_set(label) eq 0 then label = 0
if not exist(lat0) then lat0 = 0.
if not exist(lon0) then lon0 = 0.
if not exist(radang) then radang = 10.

; TODO - Include new names for crpix1, crpix2:

struct_out = struct_in

x_sec_per_pix = struct_out.cdelt1
y_sec_per_pix = struct_out.cdelt2
x_siz_img = struct_out.naxis1
y_siz_img = struct_out.naxis2
roll = struct_out.crota

sun_data = get_sun(struct_out)
r_sec = sun_data(1)
d_sun = sun_data(0)
he_lon = sun_data(10)
he_lat = sun_data(11)

r_pix1 = r_sec/x_sec_per_pix
r_pix2 = r_sec/y_sec_per_pix

; TODO - I am not sure that I don't need to subtract .5 pixel from the following:
x_lowleft2leftlimb = struct_out.crpix1 - r_pix1
x_lowleft2rightlimb = struct_out.crpix1 + r_pix1
y_lowleft2botlimb = struct_out.crpix2 - r_pix2
y_lowleft2toplimb = struct_out.crpix2 + r_pix2

x_pos_img = -x_lowleft2leftlimb > 0
y_pos_img = -y_lowleft2botlimb > 0

x_siz_win = max([x_lowleft2rightlimb,x_siz_img]) - min([x_lowleft2leftlimb,0])
y_siz_win = max([y_lowleft2toplimb,y_siz_img]) - min([y_lowleft2botlimb,0])

if keyword_set(display) then begin
  dtemp=!d.name
  set_plot,'x'
  wdef,win0,x_siz_win,y_siz_win,/ur
  tvscl,safe_log10(data_in),x_pos_img,y_pos_img
  wait,1
  set_plot,dtemp
endif

x_limit = [max([x_lowleft2leftlimb,0]),x_pos_img+x_lowleft2rightlimb]
y_limit = [max([y_lowleft2botlimb,0]),y_pos_img+y_lowleft2toplimb]
position = [x_limit(0)/float(x_siz_win),y_limit(0)/float(y_siz_win), $
	    x_limit(1)/float(x_siz_win),y_limit(1)/float(y_siz_win)]

get_ellipse, pang=roll, b0=-he_lat, lat0=lat0, lon0=lon0, radang=radang, $
  position=position, oplot=oplot, device_coords=device_coords, $
  occult=occult, display=display, /noerase, qstop=qstop

ss_inside = polyfillv(reform(device_coords(0,*)),reform(device_coords(1,*)),x_siz_img,y_siz_img)
data_roi_only = data_in*0b
if ss_inside(0) eq -1 then data_roi_only=-1 else $
   data_roi_only(ss_inside) = data_in(ss_inside)
if keyword_set(qstop) then stop

end
