pro mk_syn_sfc, index_in, index, syn_sfc, coeff_file=coeff_file, exp_dur=exp_dur
;+
;NAME: 
;         mk_syn_sfc.pro
;
;PURPOSE:
;         Create a synthetic Yohkoh-SFC for a given time and pointing,
;         as determined by the input index file.  The synthetic SFC is
;         generated from the 15x512x512 coefficient data structure generated
;         by mk_synsfc_coeff.pro and stored in DIR_SXT_SYNSFC.
;
;CALLING SEQUENCE:
;         mk_syn_sfc, index_in, index, syn_sfc
;         mk_syn_sfc, index_in, index, syn_sfc, exp_dur=exp_dur
;
;INPUT:
;         index_in - index file, or array of index files (all from
;                    a single filter failure epoch) that is used                
;                    to determine the date, x and y coordinates of  
;                    sun-center, and roll of the image.
; 
;OPTIONAL KEYWORD INPUT:
;         exp_dur -  Exposure duration for synthesized SFC.  The default
;                    exposure duration is 2691.09.         
;        
;         coeff_file - filename of synthetic coefficients to use to create
;                      synthetic SFCs.
;
;OUTPUT:  
;         index -    The index_in values modified to indecate that the file
;                    is a synthetic SFC, with an exposure time. NOT YET 
;                    IMPLEMENTED.
;
;         syn_sfc -  The synthetic SFC, a 512x512 real-number array.
;
;HISTORY:
;
;V. 1.00  Written 08-Sep-2000 (PGS)
;V. 1.01  S.L.Freeland - merge SLF history Mods into PGS 8-Jan-2001 version
;-
;Declare a common block to minimize reads of coefficient structure.
common coeff_mat, coeff_s, coeff_string
;
;Initalize relevant parameters: filter a, filter b, x, y, roll, 
;solar diameter, exposure duration
filt_a      = gt_filta(index_in)
filt_b      = gt_filtb(index_in)
xy_param    = sxt_cen(index_in, roll=roll)
x_term      = xy_param(0,*)
y_term      = xy_param(1,*)
roll_term   = roll
solard_term = (get_sun(index_in))(1,*)
IF (1-keyword_set(exp_dur)) THEN exp_dur = 2691.09; TODO! pick a better number.

;Test that all SFC being generated will be from the same
;filter failure epoch.
epoch_date  = gt_day(index_in(0),/str)
epoch_bound = [get_yo_dates(/launch),get_yo_dates(/entrance),ut_time(!stime)]
t_compare   = ssw_time_compare(epoch_bound, epoch_date, /later)
later     = where((t_compare eq 1), num_gd_later)
epoch= later(0)-2

;Test that all index records are from the same epoch.
epoch_test = gt_day(index_in) ;TEST LIMITS!!!
temp_bound=intarr(n_elements(epoch_bound))
FOR i=0,n_elements(temp_bound)-2  DO BEGIN
   x_tmp=where((epoch_test gt gt_day(epoch_bound(i))) $
      and (epoch_test lt gt_day(epoch_bound(i+1))),num_good)
   temp_bound(i) = num_good
ENDFOR
y_tmp = where(temp_bound GT 0, num_gt_zero)
IF (num_gt_zero GT 1) THEN BEGIN $
   MESSAGE, 'Index_in files must be from the SAME filter failure epoch.'
   RETURN
ENDIF

;obsolete, delete in future.
;fname = 'synsfc_' + $
;       str_replace(strtrim(gt_filtb(index_in(0),/string),2),'.','') + $
;       time2file((get_yo_dates(/ent))(epoch)) 

outdir = ''
;Construct file name of coefficient matrix to restore.
IF (1-keyword_set(coeff_file)) THEN BEGIN
   outdir=get_logenv('DIR_SXT_SYNSFC')
   fname=('synsfc_' + $
       str_replace(strtrim(gt_filta(index_in,/short),2),'.','') + $
       str_replace(strtrim(gt_filtb(index_in,/short),2),'.','') + $
       time2file((get_yo_dates(/ent))(epoch)))(0)
ENDIF

;Conditional read of coefficient data structure file, in case it's not 
;already in the common block defined above.
CASE n_elements(coeff_string) OF
	0: BEGIN
              restgenx, coeff_s, coeff_string, file=concat_dir(outdir,fname)
              message, /info, 'FIRST TIME READ of coeff_matrix file.'
           END 
        1: IF (coeff_string NE fname) THEN $
              restgenx, coeff_s, coeff_string, file=concat_dir(outdir,fname) $
              ELSE message,/info, 'coeff_matrix file NOT read.  Already exists.'
        ELSE: MESSAGE,/info, 'wrong number of terms restored from coeff_s.'
ENDCASE

;Construction of a vector of parameter vectors that are 
;derived from index_in values. (One vector for each index 
;of the index_in vector.)
param=fltarr(n_elements(index_in),15)
for k=0,n_elements(index_in)-1 do begin
   param(k, 0)  = 1. 
   param(k, 1) = x_term(0,k)
   param(k, 2) = y_term(0,k)
   param(k, 3) = roll_term(k)
   param(k, 4) = solard_term(0,k)
   param(k, 5) = x_term(0,k)^2
   param(k, 6) = y_term(0,k)^2
   param(k, 7) = roll_term(k)^2
   param(k, 8) = solard_term(0,k)^2
   param(k, 9) = x_term(0,k)*y_term(0,k)
   param(k,10) = x_term(0,k)*roll_term(k) 
   param(k,11) = y_term(0,k)*roll_term(k)
   param(k,12) = x_term(0,k)*solard_term(0,k)
   param(k,13) = y_term(0,k)*solard_term(0,k)
   param(k,14) = roll_term(k)*solard_term(0,k)
endfor

;Temporary array to hold the vector of synthetic 
;SFCs to be constructed. (One synthetic SFC for 
;each index of the index_in vector.)
temp_arr = fltarr(512,512,n_elements(index_in))

;Calculate the inner product of the 15 coefficient matrices
;with the 15-d parameter vectors derived from the index_in index
;records for each index element.
syn_sfc=0.
for i = 0, 14 do begin
   for j = 0, (n_elements(index_in)-1) do $
      temp_arr(*,*,j) = coeff_s(i).coeff_vect*param(j,i)
   syn_sfc = syn_sfc + temp_arr
endfor

;Construct vector of index records for vector of constructed
;synthetic SFCs by modifying exposure duration, and signaling
;synthetic SFC entry. CURRENTLY UNIMPLEMENTED!!! 
index = index_in ; TODO:modify fields in index_in: exp_dur, SYNTHETIC SFC.

; S.L.Freeland - 18-Sep-2000 - add some history information
his_index,index,/enable
index.sxt.imgparam=1
index.his.pixel_size=1
index.sxt.shape_cmd=[512,512]
index.sxt.shape_sav=[512,512]
index.his.expdur=exp_dur

;Give the synthetic SFC a 'pseudo' exposure time.  Default = 2691.09
syn_sfc=syn_sfc*exp_dur 



end
